# -*- coding: utf-8 -*-

"""ImgYaso
https://github.com/apachecn/imgyaso"""

from .adathres import adathres_bts, adathres
from .dither import grid_bts, grid, noise_bts, noise, noisebw_bts, noisebw
from .quant import pngquant_bts, pngquant
from .trunc import trunc_bts, trunc
from .util import *

__author__ = "ApacheCN"
__email__ = "apachecn@163.com"
__license__ = "SATA"
__version__ = "2023.1.27.3"
